﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using Syncfusion.Mvc.DocIO.Web;
using System.Drawing;
using System.IO;

namespace GenerateWord.Controllers
{
    public class HomeController : Controller
    {
        string dataPath;
        #region Generate Word
        public ActionResult Index(string Group1)
        {
            if (Group1 == null)
                return View();

            dataPath = ResolveApplicationDataPath("\\", "Data");
            //A new document is created.
            WordDocument document = new WordDocument();
            //Adding a new section to the document.
            WSection section = document.AddSection() as WSection;
            //Set Margin of the section
            section.PageSetup.Margins.All = 72;
            //Set page size of the section
            section.PageSetup.PageSize = new SizeF(612, 792);
            //Appends table to the document
            WTable table = document.LastSection.AddTable() as WTable;
            table.ResetCells(3, 3);
            //Sets the header row
            table.Rows[0].IsHeader = true;
            int rowIndex = 0; 
            float maxColumnWidth = 0;
            //Appends paragraph to the table cell
            foreach(WTableRow row in table.Rows)
            {
                foreach(WTableCell cell in row.Cells)
                {
                    if(rowIndex == 0)
                    {
                        cell.AddParagraph().AppendText("Essential DocIO");
                        rowIndex = -1;
                    }
                    else
                        cell.AddParagraph().AppendText("Cell");
                    //Sets the padding to the current cell
                    cell.CellFormat.Paddings.All = 5;       
                    //To iterate the paragraphs of the cell
                    foreach (WParagraph paragraph in cell.Paragraphs)
                    {
                        //To iterate the child entities of the paragraph
                        foreach (Entity entity in paragraph.ChildEntities)
                        {
                            //To Access the text range
                            if (entity is WTextRange)
                            {
                                //To measure the string width
                                float textWidth = MeasureString(entity as WTextRange);
                                if (textWidth > maxColumnWidth)
                                    maxColumnWidth = textWidth;
                            }
                        }
                    }
                }
            }
            //In order to preserve the space in between cell boundaries and cell text 
            maxColumnWidth += 20;
            //Resets table cell width with the max width
            foreach (WTableRow tableRow in table.Rows)
            {
                foreach (WTableCell tableCell in tableRow.Cells)
                {
                    tableCell.Width = maxColumnWidth;
                }
            }
            
            #region saveOption
            if (Group1 == "Word97To2003")
            {
                //Save as .doc Word 97-2003 format
                return document.ExportAsActionResult("Sample.doc", FormatType.Doc, HttpContext.ApplicationInstance.Response, HttpContentDisposition.Attachment);
            }
            //Save as .docx Word 2007 format
            else if (Group1 == "Word2007")
            {
                return document.ExportAsActionResult("Sample.docx", FormatType.Word2007, HttpContext.ApplicationInstance.Response, HttpContentDisposition.Attachment);

            }
            //Save as .docx Word 2010 format
            else if (Group1 == "Word2010")
            {
                return document.ExportAsActionResult("Sample.docx", FormatType.Word2010, HttpContext.ApplicationInstance.Response, HttpContentDisposition.Attachment);

            }
            //Save as .docx Word 2013 format
            else if (Group1 == "Word2013")
            {
                return document.ExportAsActionResult("Sample.docx", FormatType.Word2013, HttpContext.ApplicationInstance.Response, HttpContentDisposition.Attachment);

            }
            #endregion saveOption

            return View();
        }
        #endregion Getting Started
        /// <summary>
        /// Measures the width of the string
        /// </summary>
        /// <param name="textRange"></param>
        /// <returns></returns>
        private float MeasureString(WTextRange textRange)
        {
            Bitmap bmp = new Bitmap(1, 1);
            Graphics g = Graphics.FromImage(bmp);
            bmp.SetResolution(120, 120);
            g.PageUnit = GraphicsUnit.Point;
            StringFormat format = StringFormat.GenericTypographic;
            format.FormatFlags = System.Drawing.StringFormatFlags.FitBlackBox
                | System.Drawing.StringFormatFlags.MeasureTrailingSpaces | System.Drawing.StringFormatFlags.NoClip;
            return g.MeasureString(textRange.Text, textRange.CharacterFormat.Font, new PointF(0, 0), format).Width;
        }
        /// <summary>
        /// Data folder path is resolved from requested page physical path
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="folderName"></param>
        /// <returns></returns>
        protected string ResolveApplicationDataPath(string fileName, string folderName)
        {
            string dataPath = null;
            dataPath = Server.MapPath(@"App_Data//Images//").ToString();
            return string.Format("{0}\\{1}", dataPath, fileName);
        }
    }
}
